#! /bin/bash

# MACNICA AMERICAS
# MICKAEL TOUMI (mickael.toumi@macnica.com) 
# MADE ON 4/14/2016 - V1.0


#!!!!!!!!!!!!!!!!!!!!!!! USER CONFIGURATION SECTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

# MANDATORY PARAMETER
# Enter the full path to SDK starting with the disk letter
# For example a valid SDK path could be: 
# SDK_PATH_FROM_USER='C:\Users\mickael\Documents\WICED\WICED-Smart-SDK-2.2.2\'
SDK_PATH_FROM_USER='C:\Users\girving\Documents\WICED\WICED-Smart-SDK-2.2.3\'


# MANDATORY PARAMETER
# Set the name of the default firmware file donwloaded from: http://www.m-pression.com/solutions/boards/odyssey/odyssey-downloads
# This .hex file is the final firmware file programmed with the default BT MAC ADDRESS: C1:44:49:45:47:4F
# This file is built with the Broadcom SDK, if you need more information contact Macnica.
# Note: this variable can be overriden with a new firmware file name in case the user needs to set a specific BT MACC ADDRESS.
ODYSSEY_FIRMWARE_HEX_FILE_NAME=BLE_odyssey_v2_0_17Feb2016.hex

# OPTIONAL PARAMETER
# SET_NEW_MAC_ADDRESS=n (default configuration)
# Required: bt_mac_address.txt with one or more mac addresses in the following format (12 digits):123456789ABC 
# Required: CGS file downloaded from: http://www.m-pression.com/solutions/boards/odyssey/odyssey-downloads
#
# If set to n (no) this will download the default binary with predefined BT MAC address from Macnica. 
# If set to y (yes) this will take the first BT MAC address stored in bt_mac_address.txt
SET_NEW_MAC_ADDRESS=n
ODYSSEY_FIRMWARE_CGS_FILE_NAME=A_20737A1_BLE_v2_0_17Feb2016.cgs

#OPTIONAL PARAMETER
# BATCH_BT_MAC_ADDRESS (not implemented yet!):
# If set to y (yes) the script will try to program multiple Odyssey board (on at a time). Make sure
# to store enough BT MAC addresses in bt_mac_address.txt.
# If set to n (NO) the script will download the pre-build HEX file directly into the Odyssey.
#BATCH_BT_MAC_ADDRESS=n


#!!!!!!!!!!!!!!!!!!!!!!! DO NOT EDIT PASS THIS POINT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


# Paths needed to grab the executable from the Broadcom SDK.
DEFINE_WINDOWS_PATH_TO_PERL='WICED-Smart-SDK\Tools\common\Win32\'
DEFINE_WINDOWS_PATH_TO_PERL_SCRIPT_GET_PORT='WICED-Smart-SDK\Tools\get_com_port\'
DEFINE_WINDOWS_PATH_TO_DETECT_IDENT='WICED-Smart-SDK\Tools\DetectAndId\Win32\'
DEFINE_WINDOWS_PLATFORM_PATH='WICED-Smart-SDK\Platforms\BCM920737TAG_Q32\'

#CGS specifics
DEFINE_WINDOWS_PATH_TO_CGS='WICED-Smart-SDK\Tools\CGS\Win32\'


#Chipload specifics
DEFINE_WINDOWS_PATH_TO_CHIPLOAD='WICED-Smart-SDK\Tools\ChipLoad\Win32\'
DEFINE_WINDOWS_PATH_TO_PLATFORM_FILES='WICED-Smart-SDK\Platforms\BCM920737TAG_Q32\'


# List of required executable from the Broadcom SDK
DEFINE_WINDOWS_EXECUTABLE_PERL='perl.exe'
DEFINE_WINDOWS_EXECUTABLE_DETECT_IDENT='DetAndId.exe'
DEFINE_WINDOWS_EXECUTABLE_CHIPLOAD='ChipLoad.exe'
DEFINE_WINDOWS_EXECUTABLE_CGS='cgs.exe'

# Libraries, Perl and text files
DEFINE_WINDOWS_LIBS=*.dll
DEFINE_WINDOWS_SCRIPT_PERL_GET_PORT='get_com_port.pl'
DEFINE_WINDOWS_IDFILE='BCM20737A1_IDFILE.txt'
DEFINE_WINDOWS_BTP_FILE='20737_EEPROM.btp'
DEFINE_WINDOWS_EEPROM_PIN1='uart_DISABLE_EEPROM_WP_PIN1.hex'
DEFINE_WINDOWS_HDF='ConfigDef20732_ab.hdf'
DEFINE_WINDOWS_COM_PORT='com_port.txt'

# Full path used to copy locally executables and libraries
# We are taking that route to shelter the executable from the Windows OS that may try to do thing we don't want.
DEFINE_WINDOWS_COMPLETE_PATH_TO_PERL=\
$SDK_PATH_FROM_USER$DEFINE_WINDOWS_PATH_TO_PERL$DEFINE_WINDOWS_EXECUTABLE_PERL

DEFINE_WINDOWS_FULL_PATH_TO_LIBS=\
$SDK_PATH_FROM_USER$DEFINE_WINDOWS_PATH_TO_PERL

DEFINE_WINDOWS_FULL_PATH_TO_GET_PORT=\
$SDK_PATH_FROM_USER$DEFINE_WINDOWS_PATH_TO_PERL_SCRIPT_GET_PORT$DEFINE_WINDOWS_SCRIPT_PERL_GET_PORT

DEFINE_WINDOWS_FULL_PATH_TO_DETECT_IDENT=\
$SDK_PATH_FROM_USER$DEFINE_WINDOWS_PATH_TO_DETECT_IDENT$DEFINE_WINDOWS_EXECUTABLE_DETECT_IDENT

DEFINE_WINDOWS_FULL_PATH_TO_PLATFORM_IDFILE_PATH=\
$SDK_PATH_FROM_USER$DEFINE_WINDOWS_PLATFORM_PATH$DEFINE_WINDOWS_IDFILE

DEFINE_WINDOWS_FULL_PATH_TO_CHIPLOAD=\
$SDK_PATH_FROM_USER$DEFINE_WINDOWS_PATH_TO_CHIPLOAD$DEFINE_WINDOWS_EXECUTABLE_CHIPLOAD

DEFINE_WINDOWS_FULL_PATH_TO_CHIPLOAD_BTP=\
$SDK_PATH_FROM_USER$DEFINE_WINDOWS_PATH_TO_PLATFORM_FILES$DEFINE_WINDOWS_BTP_FILE

DEFINE_WINDOWS_FULL_PATH_TO_EEPROM_PIN1=\
$SDK_PATH_FROM_USER$DEFINE_WINDOWS_PATH_TO_PLATFORM_FILES$DEFINE_WINDOWS_EEPROM_PIN1

DEFINE_WINDOWS_FULL_PATH_TO_CGS=\
$SDK_PATH_FROM_USER$DEFINE_WINDOWS_PATH_TO_CGS$DEFINE_WINDOWS_EXECUTABLE_CGS

DEFINE_WINDOWS_FULL_PATH_TO_HDF=\
$SDK_PATH_FROM_USER$DEFINE_WINDOWS_PATH_TO_PLATFORM_FILES$DEFINE_WINDOWS_HDF

# Variable used during execution

# Keep track of error during the execution of the script. Also store the failing step so the user knows what went wrong.
EXECUTION_ERROR=0


# STEP 1: make sure we have the SDK installed based on the user configuration and copy the required files

if [ -z "$SDK_PATH_FROM_USER" ]; then

  printf "\n"
  echo "MACNICA SCRIPT CHECKER>> The Broadcom SDK path is not set!" 
  echo "MACNICA SCRIPT CHECKER>> Please edit this script and add the WINDOWS path to the Broadcom SDK"
  echo "MACNICA SCRIPT CHECKER>> The format should be similart to this: 'C:\Users\xxyyzz\Documents\WICED\WICED-Smart-SDK-x.y.z\'"
  echo "MACNICA SCRIPT CHECKER>> DONT FORGET THE BACKSLASH AT THE END!! (..\)"
  EXECUTION_ERROR=1
  
fi

if [ "$EXECUTION_ERROR" -eq 0 ]; then 

  # Copy perl, all the dll libraries and files to allow execution from Cygwin sandbox.
  cp `cygpath -u $DEFINE_WINDOWS_COMPLETE_PATH_TO_PERL` .  
  cp `cygpath -u $DEFINE_WINDOWS_FULL_PATH_TO_LIBS`$DEFINE_WINDOWS_LIBS .
  cp `cygpath -u $DEFINE_WINDOWS_FULL_PATH_TO_GET_PORT` .
  cp `cygpath -u $DEFINE_WINDOWS_FULL_PATH_TO_DETECT_IDENT` .
  cp `cygpath -u $DEFINE_WINDOWS_FULL_PATH_TO_PLATFORM_IDFILE_PATH` .
  cp `cygpath -u $DEFINE_WINDOWS_FULL_PATH_TO_CHIPLOAD` .
  cp `cygpath -u $DEFINE_WINDOWS_FULL_PATH_TO_CHIPLOAD_BTP` .
  cp `cygpath -u $DEFINE_WINDOWS_FULL_PATH_TO_EEPROM_PIN1` .
  cp `cygpath -u $DEFINE_WINDOWS_FULL_PATH_TO_CGS` .
  cp `cygpath -u $DEFINE_WINDOWS_FULL_PATH_TO_HDF` .
  
  #Check if we have an error! only will show if the SDK path is wrong, that's it.
  if [ $? -gt 0 ]; then
      printf "\n"
      echo "MACNICA SCRIPT CHECKER>> Copy ERROR files may not exist of path to SDK is incorrect"
      echo "MACNICA SCRIPT CHECKER>> The Broadcom SDK path is: $SDK_PATH_FROM_USER "
      echo "MACNICA SCRIPT CHECKER>> (make sure it is the correct path!)"
      EXECUTION_ERROR=1
  fi
  
fi

#STEP 2
if [ "$EXECUTION_ERROR" -eq 0 ]; then
  
  # See if COM port can get detected, this is the magic line.
  ./perl.exe get_com_port.pl DetAndId.exe BCM20737A1_IDFILE.txt com_port.txt

  if [ -s com_port.txt ]; then
    
    echo "MACNICA SCRIPT CHECKER>> Reading COM PORT file, Odyssey has been detected"
    
    #Take the first line only of the file (we are not considering the case where multiple
    #Odysseys are connected to USB)
    COM_PORT_ODYSSEY=$(head -n 1 com_port.txt)
    
    echo "MACNICA SCRIPT CHECKER>> COM PORT found is: $COM_PORT_ODYSSEY"
    # May be used letter if more that one Odyssey at a time.
    #while IFS='' read -r com_port || [[ -n "$com_port" ]]; do 
    #  echo "COM PORT found: $com_port" 
    #done < com_port.txt
    
  else
    printf "\n\n"
    echo "MACNICA SCRIPT CHECKER>> ERROR: The script does not find a valid Odyssey board connected to the USB port of this computer."
    echo "MACNICA SCRIPT CHECKER>> Make sure the Odyssey is properly connected to the USB and recognized by the OS."
    echo "MACNICA SCRIPT CHECKER>> You should see Silabs driver for the Odyssey board loaded in the Device Manager (Windows)."
    echo "MACNICA SCRIPT CHECKER>> Lastly double check the J4 jumper on the battery board set to position 2-3 (programming)."
    EXECUTION_ERROR=2
  
  fi
fi

 #STEP 3: figure out the BT MAC ADDRESS based on the user configuration and program the Odyssey board.
if [ "$EXECUTION_ERROR" -eq 0 ]; then
  
  if [ "$SET_NEW_MAC_ADDRESS" == n ] || [ "$SET_NEW_MAC_ADDRESS" == N ]; then
    # Provide some useful information to the user, so he know which BT MAC ADDRESS is being programmed.
    echo "MACNICA SCRIPT CHECKER>> Default BT MAC ADDRESS IS BEING USED: C1444945474F"
    echo "MACNICA SCRIPT CHECKER>> Displayed as: C1:44:49:45:47:4F on your Smart Device\n"
    #This does not do anything, just as a reminder, the BT Address is already embedded in the .hex file. 
    NEW_ODYSSEY_BT_MAC_ADDRESS=C1444945474F
      
  elif [ "$SET_NEW_MAC_ADDRESS" == y ] || [ "$SET_NEW_MAC_ADDRESS" == Y ]; then
    NEW_ODYSSEY_BT_MAC_ADDRESS=$(head -n 1 bt_mac_address.txt)
    #call the CGS binary to generate the .hex file before download.
    
    ./cgs.exe -D . \
    -O DLConfigBD_ADDRBase:$NEW_ODYSSEY_BT_MAC_ADDRESS \
    -A 0xFF000000 \
    -B 20737_EEPROM.btp \
    -I BLE_odyssey_new_bt_mac_address.hex \
    --cgs-files $ODYSSEY_FIRMWARE_CGS_FILE_NAME
    
    #Check the return value of the executable to make a decision of success or failure. 
    if [ $? -eq 0 ]; then
      echo "MACNICA SCRIPT CHECKER>> New firmware with new BT MAC ADDRESS built!"
      echo "MACNICA SCRIPT CHECKER>> The new BT MAC ADDRESS is: $NEW_ODYSSEY_BT_MAC_ADDRESS"
      ODYSSEY_FIRMWARE_HEX_FILE_NAME='BLE_odyssey_new_bt_mac_address.hex'
      
    elif [ $? -eq 1 ]; then
      echo "MACNICA SCRIPT CHECKER>> OOoops Error during firmware build"
      EXECUTION_ERROR=3
    else
      echo "MACNICA SCRIPT CHECKER>> You should not be here!"
      EXECUTION_ERROR=3
    fi 
  
  else
    echo "MACNICA SCRIPT CHECKER>> You should not be here!"
    EXECUTION_ERROR=3
  fi
  
fi

#STEP 4 Ready to download, we should have the proper .hex file stored locally
if [ "$EXECUTION_ERROR" -eq 0 ]; then

    echo "MACNICA SCRIPT CHECKER>> The firmware file used to program is: $ODYSSEY_FIRMWARE_HEX_FILE_NAME"
    
   ./Chipload.exe -BLUETOOLMODE \
   -BAUDRATE 115200nfc \
   -PORT $COM_PORT_ODYSSEY \
   -MINIDRIVER uart_DISABLE_EEPROM_WP_PIN1.hex \
   -CONFIG $ODYSSEY_FIRMWARE_HEX_FILE_NAME \
   -BTP 20737_EEPROM.btp \
   -CHECKCRC \
   -NOVERIFY \
   -DLMINIDRIVERCHUNKSIZE 251
   
   #Check the return value of the executable to make a decision of success or failure. 
   if [ $? -eq 0 ]; then
     echo "MACNICA SCRIPT CHECKER>> SUCCESS! you just updated Odyssey properly"
     echo "MACNICA SCRIPT CHECKER>> The BT MAC ADDRESS is: $NEW_ODYSSEY_BT_MAC_ADDRESS"
      
   elif [ $? -eq 1 ]; then
     echo "MACNICA SCRIPT CHECKER>> OOoops Error during download procedure"
     EXECUTION_ERROR=4
   else
     echo "MACNICA SCRIPT CHECKER>> You should not be here!"
     EXECUTION_ERROR=4
   fi
fi

if [ "$EXECUTION_ERROR" -gt 0 ]; then
  
  printf "\n\n"
  echo "MACNICA SCRIPT CHECKER>> ERROR during execution of the script at STEP: $EXECUTION_ERROR"
fi



if [ "$EXECUTION_ERROR" != 1 ]; then

  echo "MACNICA SCRIPT CHECKER>> Cleaning up now..."
  
  #Clean-up the binaries that have been copied.
  rm $DEFINE_WINDOWS_EXECUTABLE_PERL \
  $DEFINE_WINDOWS_EXECUTABLE_DETECT_IDENT \
  $DEFINE_WINDOWS_EXECUTABLE_CHIPLOAD \
  $DEFINE_WINDOWS_EXECUTABLE_CGS \
  $DEFINE_WINDOWS_LIBS \
  $DEFINE_WINDOWS_SCRIPT_PERL_GET_PORT \
  $DEFINE_WINDOWS_IDFILE \
  $DEFINE_WINDOWS_BTP_FILE \
  $DEFINE_WINDOWS_EEPROM_PIN1 \
  $DEFINE_WINDOWS_HDF \
  $DEFINE_WINDOWS_COM_PORT

fi
